@@ -185,10 +185,11 @@ func Check2[U, V any](forall Seq2[U, V]) Seq2[U, V] {
 	return func(body func(U, V) bool) {
 		state := READY
 		forall(func(u U, v V) bool {
-			if state != READY {
-				panic(fail[state])
-			}
+			tmp := state
 			state = PANIC
+			if tmp != READY {
+				panic(fail[tmp])
+			}
 			ret := body(u, v)
 			if ret {
 				state = READY
@@ -208,10 +209,11 @@ func Check[U any](forall Seq[U]) Seq[U] {
 	return func(body func(U) bool) {
 		state := READY
 		forall(func(u U) bool {
-			if state != READY {
-				panic(fail[state])
-			}
+			tmp := state
 			state = PANIC
+			if tmp != READY {
+				panic(fail[tmp])
+			}
 			ret := body(u)
 			if ret {
 				state = READY
@@ -285,6 +287,26 @@ var fail []error = []error{
 	errorString(CERR_MISSING),
 }
 
+// TestNoVars ensures that versions of rangefunc that use zero or one
+// iteration variable (instead of two) run the proper number of times
+// and in the one variable case supply the proper values.
+// For #65236.
+func TestNoVars(t *testing.T) {
+	i, k := 0, 0
+	for range Check2(OfSliceIndex([]int{1, 2, 3, 4, 5, 6, 7, 8, 9, 10})) {
+		i++
+	}
+	for j := range Check2(OfSliceIndex([]int{1, 2, 3, 4, 5, 6, 7, 8, 9, 10})) {
+		k += j
+	}
+	if i != 10 {
+		t.Errorf("Expected 10, got %d", i)
+	}
+	if k != 45 {
+		t.Errorf("Expected 45, got %d", k)
+	}
+}
+
 func TestCheck(t *testing.T) {
 	i := 0
 	defer func() {
@@ -1102,12 +1124,12 @@ func TestPanickyIterator1(t *testing.T) {
 			} else {
 				t.Errorf("Saw wrong panic '%v'", r)
 			}
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
 		} else {
 			t.Errorf("Wanted to see a failure, result was %v", result)
 		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
-		}
 	}()
 	for _, z := range PanickyOfSliceIndex([]int{1, 2, 3, 4}) {
 		result = append(result, z)
@@ -1152,12 +1174,12 @@ func TestPanickyIterator2(t *testing.T) {
 			} else {
 				t.Errorf("Saw wrong panic '%v'", r)
 			}
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
 		} else {
 			t.Errorf("Wanted to see a failure, result was %v", result)
 		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
-		}
 	}()
 	for _, x := range OfSliceIndex([]int{100, 200}) {
 		result = append(result, x)
@@ -1187,11 +1209,11 @@ func TestPanickyIterator2Check(t *testing.T) {
 			} else {
 				t.Errorf("Saw wrong panic '%v'", r)
 			}
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
 		} else {
-			t.Errorf("Wanted to see a failure, result was %v", result)
-		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
+			t.Errorf("Wanted to see a panic, result was %v", result)
 		}
 	}()
 	for _, x := range Check2(OfSliceIndex([]int{100, 200})) {
@@ -1214,13 +1236,19 @@ func TestPanickyIterator2Check(t *testing.T) {
 
 func TestPanickyIterator3(t *testing.T) {
 	var result []int
-	var expect = []int{100, 10, 1, 2, 200, 10, 1, 2}
+	var expect = []int{100, 10, 1, 2}
 	defer func() {
 		if r := recover(); r != nil {
-			t.Errorf("Unexpected panic '%v'", r)
-		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
+			if matchError(r, RERR_MISSING) {
+				t.Logf("Saw expected panic '%v'", r)
+			} else {
+				t.Errorf("Saw wrong panic '%v'", r)
+			}
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
+		} else {
+			t.Errorf("Wanted to see a panic, result was %v", result)
 		}
 	}()
 	for _, x := range OfSliceIndex([]int{100, 200}) {
@@ -1242,13 +1270,19 @@ func TestPanickyIterator3(t *testing.T) {
 }
 func TestPanickyIterator3Check(t *testing.T) {
 	var result []int
-	var expect = []int{100, 10, 1, 2, 200, 10, 1, 2}
+	var expect = []int{100, 10, 1, 2}
 	defer func() {
 		if r := recover(); r != nil {
-			t.Errorf("Unexpected panic '%v'", r)
-		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
+			if matchError(r, CERR_MISSING) {
+				t.Logf("Saw expected panic '%v'", r)
+			} else {
+				t.Errorf("Saw wrong panic '%v'", r)
+			}
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
+		} else {
+			t.Errorf("Wanted to see a panic, result was %v", result)
 		}
 	}()
 	for _, x := range Check2(OfSliceIndex([]int{100, 200})) {
@@ -1278,9 +1312,11 @@ func TestPanickyIterator4(t *testing.T) {
 			} else {
 				t.Errorf("Saw wrong panic '%v'", r)
 			}
-		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
+		} else {
+			t.Errorf("Wanted to see a panic, result was %v", result)
 		}
 	}()
 	for _, x := range SwallowPanicOfSliceIndex([]int{1, 2, 3, 4}) {
@@ -1301,9 +1337,11 @@ func TestPanickyIterator4Check(t *testing.T) {
 			} else {
 				t.Errorf("Saw wrong panic '%v'", r)
 			}
-		}
-		if !slices.Equal(expect, result) {
-			t.Errorf("Expected %v, got %v", expect, result)
+			if !slices.Equal(expect, result) {
+				t.Errorf("Expected %v, got %v", expect, result)
+			}
+		} else {
+			t.Errorf("Wanted to see a panic, result was %v", result)
 		}
 	}()
 	for _, x := range Check2(SwallowPanicOfSliceIndex([]int{1, 2, 3, 4})) {
@@ -1389,33 +1427,76 @@ X:
 
 // TestVeryBad1 checks the behavior of an extremely poorly behaved iterator.
 func TestVeryBad1(t *testing.T) {
-	result := veryBad([]int{10, 20, 30, 40, 50}) // odd length
-	expect := []int{1, 10}
+	expect := []int{} // assignment does not happen
+	var result []int
 
-	if !slices.Equal(expect, result) {
-		t.Errorf("Expected %v, got %v", expect, result)
+	defer func() {
+		if r := recover(); r != nil {
+			expectPanic(t, r, RERR_MISSING)
+			if !slices.Equal(expect, result) {
+				t.Errorf("(Inner) Expected %v, got %v", expect, result)
+			}
+		} else {
+			t.Error("Wanted to see a failure")
+		}
+	}()
+
+	result = veryBad([]int{10, 20, 30, 40, 50}) // odd length
+
+}
+
+func expectPanic(t *testing.T, r any, s string) {
+	if matchError(r, s) {
+		t.Logf("Saw expected panic '%v'", r)
+	} else {
+		t.Errorf("Saw wrong panic '%v'", r)
+	}
+}
+
+func expectError(t *testing.T, err any, s string) {
+	if matchError(err, s) {
+		t.Logf("Saw expected error '%v'", err)
+	} else {
+		t.Errorf("Saw wrong error '%v'", err)
 	}
 }
 
 // TestVeryBad2 checks the behavior of an extremely poorly behaved iterator.
 func TestVeryBad2(t *testing.T) {
-	result := veryBad([]int{10, 20, 30, 40}) // even length
-	expect := []int{1, 10}
+	result := []int{}
+	expect := []int{}
+
+	defer func() {
+		if r := recover(); r != nil {
+			expectPanic(t, r, RERR_MISSING)
+			if !slices.Equal(expect, result) {
+				t.Errorf("(Inner) Expected %v, got %v", expect, result)
+			}
+		} else {
+			t.Error("Wanted to see a failure")
+		}
+	}()
+
+	result = veryBad([]int{10, 20, 30, 40}) // even length
 
-	if !slices.Equal(expect, result) {
-		t.Errorf("Expected %v, got %v", expect, result)
-	}
 }
 
 // TestVeryBadCheck checks the behavior of an extremely poorly behaved iterator,
 // which also suppresses the exceptions from "Check"
 func TestVeryBadCheck(t *testing.T) {
-	result := veryBadCheck([]int{10, 20, 30, 40}) // even length
-	expect := []int{1, 10}
+	expect := []int{}
+	var result []int
+	defer func() {
+		if r := recover(); r != nil {
+			expectPanic(t, r, CERR_MISSING)
+		}
+		if !slices.Equal(expect, result) {
+			t.Errorf("Expected %v, got %v", expect, result)
+		}
+	}()
+
+	result = veryBadCheck([]int{10, 20, 30, 40}) // even length
 
-	if !slices.Equal(expect, result) {
-		t.Errorf("Expected %v, got %v", expect, result)
-	}
 }
 
 // TestOk is the nice version of the very bad iterator.
@@ -2099,3 +2180,27 @@ func TestTwoLevelReturnCheck(t *testing.T) {
 		t.Errorf("Expected y=3, got y=%d\n", y)
 	}
 }
+
+func Bug70035(s1, s2, s3 []string) string {
+	var c1 string
+	for v1 := range slices.Values(s1) {
+		var c2 string
+		for v2 := range slices.Values(s2) {
+			var c3 string
+			for v3 := range slices.Values(s3) {
+				c3 = c3 + v3
+			}
+			c2 = c2 + v2 + c3
+		}
+		c1 = c1 + v1 + c2
+	}
+	return c1
+}
+
+func Test70035(t *testing.T) {
+	got := Bug70035([]string{"1", "2", "3"}, []string{"a", "b", "c"}, []string{"A", "B", "C"})
+	want := "1aABCbABCcABC2aABCbABCcABC3aABCbABCcABC"
+	if got != want {
+		t.Errorf("got %v, want %v", got, want)
+	}
+}
