@@ -7,12 +7,12 @@ package modload
 import (
 	"bytes"
 	"context"
-	"encoding/json"
 	"errors"
 	"fmt"
 	"internal/godebugs"
 	"internal/lazyregexp"
 	"io"
+	"maps"
 	"os"
 	"path"
 	"path/filepath"
@@ -23,6 +23,7 @@ import (
 
 	"cmd/go/internal/base"
 	"cmd/go/internal/cfg"
+	"cmd/go/internal/fips140"
 	"cmd/go/internal/fsys"
 	"cmd/go/internal/gover"
 	"cmd/go/internal/lockedfile"
@@ -81,6 +82,36 @@ func EnterModule(ctx context.Context, enterModroot string) {
 	LoadModFile(ctx)
 }
 
+// EnterWorkspace enters workspace mode from module mode, applying the updated requirements to the main
+// module to that module in the workspace. There should be no calls to any of the exported
+// functions of the modload package running concurrently with a call to EnterWorkspace as
+// EnterWorkspace will modify the global state they depend on in a non-thread-safe way.
+func EnterWorkspace(ctx context.Context) (exit func(), err error) {
+	// Find the identity of the main module that will be updated before we reset modload state.
+	mm := MainModules.mustGetSingleMainModule()
+	// Get the updated modfile we will use for that module.
+	_, _, updatedmodfile, err := UpdateGoModFromReqs(ctx, WriteOpts{})
+	if err != nil {
+		return nil, err
+	}
+
+	// Reset the state to a clean state.
+	oldstate := setState(state{})
+	ForceUseModules = true
+
+	// Load in workspace mode.
+	InitWorkfile()
+	LoadModFile(ctx)
+
+	// Update the content of the previous main module, and recompute the requirements.
+	*MainModules.ModFile(mm) = *updatedmodfile
+	requirements = requirementsFromModFiles(ctx, MainModules.workFile, slices.Collect(maps.Values(MainModules.modFiles)), nil)
+
+	return func() {
+		setState(oldstate)
+	}, nil
+}
+
 // Variable set in InitWorkfile
 var (
 	// Set to the path to the go.work file, or "" if workspace mode is disabled.
@@ -108,6 +139,8 @@ type MainModuleSet struct {
 
 	modFiles map[module.Version]*modfile.File
 
+	tools map[string]bool
+
 	modContainingCWD module.Version
 
 	workFile *modfile.WorkFile
@@ -135,6 +168,15 @@ func (mms *MainModuleSet) Versions() []module.Version {
 	return mms.versions
 }
 
+// Tools returns the tools defined by all the main modules.
+// The key is the absolute package path of the tool.
+func (mms *MainModuleSet) Tools() map[string]bool {
+	if mms == nil {
+		return nil
+	}
+	return mms.tools
+}
+
 func (mms *MainModuleSet) Contains(path string) bool {
 	if mms == nil {
 		return false
@@ -344,7 +386,8 @@ func BinDir() string {
 // for example 'go mod tidy', that don't operate in workspace mode.
 func InitWorkfile() {
 	// Initialize fsys early because we need overlay to read go.work file.
-	if err := fsys.Init(base.Cwd()); err != nil {
+	fips140.Init()
+	if err := fsys.Init(); err != nil {
 		base.Fatal(err)
 	}
 	workFilePath = FindGoWork(base.Cwd())
@@ -382,15 +425,44 @@ func WorkFilePath() string {
 // Reset clears all the initialized, cached state about the use of modules,
 // so that we can start over.
 func Reset() {
-	initialized = false
-	ForceUseModules = false
-	RootMode = 0
-	modRoots = nil
-	cfg.ModulesEnabled = false
-	MainModules = nil
-	requirements = nil
-	workFilePath = ""
-	modfetch.Reset()
+	setState(state{})
+}
+
+func setState(s state) state {
+	oldState := state{
+		initialized:     initialized,
+		forceUseModules: ForceUseModules,
+		rootMode:        RootMode,
+		modRoots:        modRoots,
+		modulesEnabled:  cfg.ModulesEnabled,
+		mainModules:     MainModules,
+		requirements:    requirements,
+	}
+	initialized = s.initialized
+	ForceUseModules = s.forceUseModules
+	RootMode = s.rootMode
+	modRoots = s.modRoots
+	cfg.ModulesEnabled = s.modulesEnabled
+	MainModules = s.mainModules
+	requirements = s.requirements
+	workFilePath = s.workFilePath
+	// The modfetch package's global state is used to compute
+	// the go.sum file, so save and restore it along with the
+	// modload state.
+	oldState.modfetchState = modfetch.SetState(s.modfetchState)
+	return oldState
+}
+
+type state struct {
+	initialized     bool
+	forceUseModules bool
+	rootMode        Root
+	modRoots        []string
+	modulesEnabled  bool
+	mainModules     *MainModuleSet
+	requirements    *Requirements
+	workFilePath    string
+	modfetchState   modfetch.State
 }
 
 // Init determines whether module mode is enabled, locates the root of the
@@ -403,6 +475,8 @@ func Init() {
 	}
 	initialized = true
 
+	fips140.Init()
+
 	// Keep in sync with WillBeEnabled. We perform extra validation here, and
 	// there are lots of diagnostics and side effects, so we can't use
 	// WillBeEnabled directly.
@@ -423,7 +497,7 @@ func Init() {
 		return
 	}
 
-	if err := fsys.Init(base.Cwd()); err != nil {
+	if err := fsys.Init(); err != nil {
 		base.Fatal(err)
 	}
 
@@ -437,23 +511,6 @@ func Init() {
 		os.Setenv("GIT_TERMINAL_PROMPT", "0")
 	}
 
-	// Disable any ssh connection pooling by Git.
-	// If a Git subprocess forks a child into the background to cache a new connection,
-	// that child keeps stdout/stderr open. After the Git subprocess exits,
-	// os/exec expects to be able to read from the stdout/stderr pipe
-	// until EOF to get all the data that the Git subprocess wrote before exiting.
-	// The EOF doesn't come until the child exits too, because the child
-	// is holding the write end of the pipe.
-	// This is unfortunate, but it has come up at least twice
-	// (see golang.org/issue/13453 and golang.org/issue/16104)
-	// and confuses users when it does.
-	// If the user has explicitly set GIT_SSH or GIT_SSH_COMMAND,
-	// assume they know what they are doing and don't step on it.
-	// But default to turning off ControlMaster.
-	if os.Getenv("GIT_SSH") == "" && os.Getenv("GIT_SSH_COMMAND") == "" {
-		os.Setenv("GIT_SSH_COMMAND", "ssh -o ControlMaster=no -o BatchMode=yes")
-	}
-
 	if os.Getenv("GCM_INTERACTIVE") == "" {
 		os.Setenv("GCM_INTERACTIVE", "never")
 	}
@@ -612,7 +669,7 @@ func inWorkspaceMode() bool {
 	return workFilePath != ""
 }
 
-// HasModRoot reports whether a main module is present.
+// HasModRoot reports whether a main module or main modules are present.
 // HasModRoot may return false even if Enabled returns true: for example, 'get'
 // does not require a main module.
 func HasModRoot() bool {
@@ -638,6 +695,9 @@ func ModFilePath() string {
 }
 
 func modFilePath(modRoot string) string {
+	// TODO(matloob): This seems incompatible with workspaces
+	// (unless the user's intention is to replace all workspace modules' modfiles?).
+	// Should we produce an error in workspace mode if cfg.ModFile is set?
 	if cfg.ModFile != "" {
 		return cfg.ModFile
 	}
@@ -648,24 +708,34 @@ func die() {
 	if cfg.Getenv("GO111MODULE") == "off" {
 		base.Fatalf("go: modules disabled by GO111MODULE=off; see 'go help modules'")
 	}
-	if inWorkspaceMode() {
-		base.Fatalf("go: no modules were found in the current workspace; see 'go help work'")
-	}
-	if dir, name := findAltConfig(base.Cwd()); dir != "" {
-		rel, err := filepath.Rel(base.Cwd(), dir)
-		if err != nil {
-			rel = dir
+	if !inWorkspaceMode() {
+		if dir, name := findAltConfig(base.Cwd()); dir != "" {
+			rel, err := filepath.Rel(base.Cwd(), dir)
+			if err != nil {
+				rel = dir
+			}
+			cdCmd := ""
+			if rel != "." {
+				cdCmd = fmt.Sprintf("cd %s && ", rel)
+			}
+			base.Fatalf("go: cannot find main module, but found %s in %s\n\tto create a module there, run:\n\t%sgo mod init", name, dir, cdCmd)
 		}
-		cdCmd := ""
-		if rel != "." {
-			cdCmd = fmt.Sprintf("cd %s && ", rel)
-		}
-		base.Fatalf("go: cannot find main module, but found %s in %s\n\tto create a module there, run:\n\t%sgo mod init", name, dir, cdCmd)
 	}
 	base.Fatal(ErrNoModRoot)
 }
 
-var ErrNoModRoot = errors.New("go.mod file not found in current directory or any parent directory; see 'go help modules'")
+// noMainModulesError returns the appropriate error if there is no main module or
+// main modules depending on whether the go command is in workspace mode.
+type noMainModulesError struct{}
+
+func (e noMainModulesError) Error() string {
+	if inWorkspaceMode() {
+		return "no modules were found in the current workspace; see 'go help work'"
+	}
+	return "go.mod file not found in current directory or any parent directory; see 'go help modules'"
+}
+
+var ErrNoModRoot noMainModulesError
 
 type goModDirtyError struct{}
 
@@ -681,7 +751,10 @@ func (goModDirtyError) Error() string {
 
 var errGoModDirty error = goModDirtyError{}
 
-func loadWorkFile(path string) (workFile *modfile.WorkFile, modRoots []string, err error) {
+// LoadWorkFile parses and checks the go.work file at the given path,
+// and returns the absolute paths of the workspace modules' modroots.
+// It does not modify the global state of the modload package.
+func LoadWorkFile(path string) (workFile *modfile.WorkFile, modRoots []string, err error) {
 	workDir := filepath.Dir(path)
 	wf, err := ReadWorkFile(path)
 	if err != nil {
@@ -747,34 +820,25 @@ func UpdateWorkGoVersion(wf *modfile.WorkFile, goVers string) (changed bool) {
 
 	wf.AddGoStmt(goVers)
 
-	// We wrote a new go line. For reproducibility,
-	// if the toolchain running right now is newer than the new toolchain line,
-	// update the toolchain line to record the newer toolchain.
-	// The user never sets the toolchain explicitly in a 'go work' command,
-	// so this is only happening as a result of a go or toolchain line found
-	// in a module.
-	// If the toolchain running right now is a dev toolchain (like "go1.21")
-	// writing 'toolchain go1.21' will not be useful, since that's not an actual
-	// toolchain you can download and run. In that case fall back to at least
-	// checking that the toolchain is new enough for the Go version.
-	toolchain := "go" + old
-	if wf.Toolchain != nil {
-		toolchain = wf.Toolchain.Name
-	}
-	if gover.IsLang(gover.Local()) {
-		toolchain = gover.ToolchainMax(toolchain, "go"+goVers)
-	} else {
-		toolchain = gover.ToolchainMax(toolchain, "go"+gover.Local())
+	if wf.Toolchain == nil {
+		return true
 	}
 
-	// Drop the toolchain line if it is implied by the go line
+	// Drop the toolchain line if it is implied by the go line,
+	// if its version is older than the version in the go line,
 	// or if it is asking for a toolchain older than Go 1.21,
 	// which will not understand the toolchain line.
-	if toolchain == "go"+goVers || gover.Compare(gover.FromToolchain(toolchain), gover.GoStrictVersion) < 0 {
+	// Previously, a toolchain line set to the local toolchain
+	// version was added so that future operations on the go file
+	// would use the same toolchain logic for reproducibility.
+	// This behavior seemed to cause user confusion without much
+	// benefit so it was removed. See #65847.
+	toolchain := wf.Toolchain.Name
+	toolVers := gover.FromToolchain(toolchain)
+	if toolchain == "go"+goVers || gover.Compare(toolVers, goVers) < 0 || gover.Compare(toolVers, gover.GoStrictVersion) < 0 {
 		wf.DropToolchainStmt()
-	} else {
-		wf.AddToolchainStmt(toolchain)
 	}
+
 	return true
 }
 
@@ -839,7 +903,7 @@ func loadModFile(ctx context.Context, opts *PackageOpts) (*Requirements, error)
 	var workFile *modfile.WorkFile
 	if inWorkspaceMode() {
 		var err error
-		workFile, modRoots, err = loadWorkFile(workFilePath)
+		workFile, modRoots, err = LoadWorkFile(workFilePath)
 		if err != nil {
 			return nil, err
 		}
@@ -929,7 +993,7 @@ func loadModFile(ctx context.Context, opts *PackageOpts) (*Requirements, error)
 					err = errWorkTooOld(gomod, workFile, tooNew.GoVersion)
 				} else {
 					err = fmt.Errorf("cannot load module %s listed in go.work file: %w",
-						base.ShortPath(filepath.Dir(gomod)), err)
+						base.ShortPath(filepath.Dir(gomod)), base.ShortPathError(err))
 				}
 			}
 			errs = append(errs, err)
@@ -982,10 +1046,11 @@ func loadModFile(ctx context.Context, opts *PackageOpts) (*Requirements, error)
 
 	if cfg.BuildMod == "vendor" {
 		readVendorList(VendorDir())
-		var indexes []*modFileIndex
-		var modFiles []*modfile.File
-		var modRoots []string
-		for _, m := range MainModules.Versions() {
+		versions := MainModules.Versions()
+		indexes := make([]*modFileIndex, 0, len(versions))
+		modFiles := make([]*modfile.File, 0, len(versions))
+		modRoots := make([]string, 0, len(versions))
+		for _, m := range versions {
 			indexes = append(indexes, MainModules.Index(m))
 			modFiles = append(modFiles, MainModules.ModFile(m))
 			modRoots = append(modRoots, MainModules.ModRoot(m))
@@ -1219,6 +1284,7 @@ func makeMainModules(ms []module.Version, rootDirs []string, modFiles []*modfile
 		modFiles:        map[module.Version]*modfile.File{},
 		indices:         map[module.Version]*modFileIndex{},
 		highestReplaced: map[string]string{},
+		tools:           map[string]bool{},
 		workFile:        workFile,
 	}
 	var workFileReplaces []*modfile.Replace
@@ -1301,6 +1367,17 @@ func makeMainModules(ms []module.Version, rootDirs []string, modFiles []*modfile
 					mainModules.highestReplaced[r.Old.Path] = r.Old.Version
 				}
 			}
+
+			for _, t := range modFiles[i].Tool {
+				if err := module.CheckImportPath(t.Path); err != nil {
+					if e, ok := err.(*module.InvalidPathError); ok {
+						e.Kind = "tool"
+					}
+					base.Fatal(err)
+				}
+
+				mainModules.tools[t.Path] = true
+			}
 		}
 	}
 
@@ -1504,7 +1581,7 @@ func setDefaultBuildMod() {
 					}
 				}
 			} else {
-				cfg.BuildModReason = fmt.Sprintf("Go version in " + versionSource + " is unspecified, so vendor directory was not used.")
+				cfg.BuildModReason = fmt.Sprintf("Go version in %s is unspecified, so vendor directory was not used.", versionSource)
 			}
 		}
 	}
@@ -1669,22 +1746,6 @@ func findModulePath(dir string) (string, error) {
 		}
 	}
 
-	// Look for Godeps.json declaring import path.
-	data, _ := os.ReadFile(filepath.Join(dir, "Godeps/Godeps.json"))
-	var cfg1 struct{ ImportPath string }
-	json.Unmarshal(data, &cfg1)
-	if cfg1.ImportPath != "" {
-		return cfg1.ImportPath, nil
-	}
-
-	// Look for vendor.json declaring import path.
-	data, _ = os.ReadFile(filepath.Join(dir, "vendor/vendor.json"))
-	var cfg2 struct{ RootPath string }
-	json.Unmarshal(data, &cfg2)
-	if cfg2.RootPath != "" {
-		return cfg2.RootPath, nil
-	}
-
 	// Look for path in GOPATH.
 	var badPathErr error
 	for _, gpdir := range filepath.SplitList(cfg.BuildContext.GOPATH) {
@@ -1744,6 +1805,9 @@ type WriteOpts struct {
 	DropToolchain     bool // go get toolchain@none
 	ExplicitToolchain bool // go get has set explicit toolchain version
 
+	AddTools  []string // go get -tool example.com/m1
+	DropTools []string // go get -tool example.com/m1@none
+
 	// TODO(bcmills): Make 'go mod tidy' update the go version in the Requirements
 	// instead of writing directly to the modfile.File
 	TidyWroteGo bool // Go.Version field already updated by 'go mod tidy'
@@ -1819,22 +1883,7 @@ func UpdateGoModFromReqs(ctx context.Context, opts WriteOpts) (before, after []b
 		toolchain = "go" + goVersion
 	}
 
-	// For reproducibility, if we are writing a new go line,
-	// and we're not explicitly modifying the toolchain line with 'go get toolchain@something',
-	// and the go version is one that supports switching toolchains,
-	// and the toolchain running right now is newer than the current toolchain line,
-	// then update the toolchain line to record the newer toolchain.
-	//
-	// TODO(#57001): This condition feels too complicated. Can we simplify it?
-	// TODO(#57001): Add more tests for toolchain lines.
 	toolVers := gover.FromToolchain(toolchain)
-	if wroteGo && !opts.DropToolchain && !opts.ExplicitToolchain &&
-		gover.Compare(goVersion, gover.GoStrictVersion) >= 0 &&
-		(gover.Compare(gover.Local(), toolVers) > 0 && !gover.IsLang(gover.Local())) {
-		toolchain = "go" + gover.Local()
-		toolVers = gover.FromToolchain(toolchain)
-	}
-
 	if opts.DropToolchain || toolchain == "go"+goVersion || (gover.Compare(toolVers, gover.GoStrictVersion) < 0 && !opts.ExplicitToolchain) {
 		// go get toolchain@none or toolchain matches go line or isn't valid; drop it.
 		// TODO(#57001): 'go get' should reject explicit toolchains below GoStrictVersion.
@@ -1843,6 +1892,14 @@ func UpdateGoModFromReqs(ctx context.Context, opts WriteOpts) (before, after []b
 		modFile.AddToolchainStmt(toolchain)
 	}
 
+	for _, path := range opts.AddTools {
+		modFile.AddTool(path)
+	}
+
+	for _, path := range opts.DropTools {
+		modFile.DropTool(path)
+	}
+
 	// Update require blocks.
 	if gover.Compare(goVersion, gover.SeparateIndirectVersion) < 0 {
 		modFile.SetRequire(list)
@@ -1881,7 +1938,7 @@ func commitRequirements(ctx context.Context, opts WriteOpts) (err error) {
 	}
 
 	index := MainModules.GetSingleIndexOrNil()
-	dirty := index.modFileIsDirty(modFile)
+	dirty := index.modFileIsDirty(modFile) || len(opts.DropTools) > 0 || len(opts.AddTools) > 0
 	if dirty && cfg.BuildMod != "mod" {
 		// If we're about to fail due to -mod=readonly,
 		// prefer to report a dirty go.mod over a dirty go.sum
@@ -1903,7 +1960,7 @@ func commitRequirements(ctx context.Context, opts WriteOpts) (err error) {
 
 	mainModule := MainModules.mustGetSingleMainModule()
 	modFilePath := modFilePath(MainModules.ModRoot(mainModule))
-	if _, ok := fsys.OverlayPath(modFilePath); ok {
+	if fsys.Replaced(modFilePath) {
 		if dirty {
 			return errors.New("updates to go.mod needed, but go.mod is part of the overlay specified with -overlay")
 		}
